﻿using System;
using UnityEngine;

namespace Igloo
{

    /// <summary>
    /// Default Igloo Settings configurations
    /// </summary>
    public static class DefaultConfigurations
    {
        /// <summary>
        /// Standard Equirectangular Cylinder System settings
        /// </summary>
        /// <returns>A populated settings file for an Equirectangular Cylinder based igloo</returns>
        public static Settings Shipping()
        {

            Settings settings = new Settings();

            NetworkSettings networkSettings = new NetworkSettings
            {
                inPort = 9007,
                outPort = 9001,
                outIP = "127.0.0.1"
            };

            settings.NetworkSettings = networkSettings;

            HeadSettings headSettings = new HeadSettings
            {
                headRotationOffset = new Vector3Item(Vector3.zero),
                headPositionOffset = new Vector3Item(0, 0, 0),
                headtracking = false,
                optitrackServerIP = "127.0.0.1",
                optitrackHeadRigidBodyID = 1
            };

            settings.HeadSettings = headSettings;


            PlayerSettings playerSettings = new PlayerSettings
            {
                Name = "player",
                usePlayer = true,
                rotationMode = 0,
                movementInput = 0,
                movementMode = 2,
                runSpeed = 10,
                walkSpeed = 5,
                smoothTime = 10,
                isCrosshair3D = false,
                crosshairHideMode = 0,
                optitrackControllerRigidBodyID = 2
            };

            settings.PlayerSettings = playerSettings;


            UISettings uiSettings = new UISettings
            {
                useUI = false,
                screenName = "Cylinder",
                screenPos = new Vector3Item(Vector3.zero),
                screenRot = new Vector3Item(Vector3.zero),
                screenScale = new Vector3Item(6.0f, 2.1f, 6.0f),
                followCrosshair = true,
                followSpeed = 10,
                debugUIMode = false
            };

            settings.UISettings = uiSettings;

            TouchScreenSettings touchScreenSettings = new TouchScreenSettings
            {
                UseTouchScreen = false,
                XPositionStart = 1.0f,
                XPositionEnd = 0.0f,
                YPositionStart = 1.0f,
                YPositionEnd = 0.0f
            };

            settings.TouchScreenSettings = touchScreenSettings;

            return settings;
        }

    }

}
