using Igloo;
using System;
using System.Collections.Generic;
using UnityEngine;

namespace Igloo
{
    [System.Serializable]
    public class Face
    {
        public string face;
        public bool enabled;
    }

    [System.Serializable]
    public enum IglooCameraType {CUBEMAP, OFFAXIS}
    [System.Serializable]
    public class IglooCameraData {

        public bool isEnabled           = true;
        public bool isStereo            = false;
        public int FPSTarget            = 30;
        public bool isSpoutOutput       = true;
        public bool isNDIOutput         = false;
        public string outputName        = "IglooUnity";
        public bool useWarpAndBlend     = false;
        public float nearClipPlane      = 0.1f;
        public float farClipPlane       = 1000f;       
        public float stereoEyeDistance  = 0.05f;

        public Vector3 cameraRotationOffset = Vector3.zero;
        public CameraTypeData cameraTypeData = new CameraTypeData();


        public CubemapCameraData CubemapData = new CubemapCameraData();
        [System.Serializable]
        public class CubemapCameraData
        {
            public bool useTruePerspective    = false;
            public int cubemapFaceResolution  = 2000;

            public Dictionary<Igloo.CubemapCamera.IglooCubemapFace, bool> faces = new Dictionary<Igloo.CubemapCamera.IglooCubemapFace, bool>()
            {
                { Igloo.CubemapCamera.IglooCubemapFace.Back , true },
                { Igloo.CubemapCamera.IglooCubemapFace.Left , true },
                { Igloo.CubemapCamera.IglooCubemapFace.Front, true },
                { Igloo.CubemapCamera.IglooCubemapFace.Right, true },
                { Igloo.CubemapCamera.IglooCubemapFace.Down , true },
                { Igloo.CubemapCamera.IglooCubemapFace.Up   , true }
            };

            public List<Face> faceList = new List<Face>();

        }

        [System.Serializable]
        public class CameraTypeData
        {
            public string doc = "0 = Cubemap Camera, 1 = Off Axis Camera";
            public IglooCameraType cameraType = IglooCameraType.CUBEMAP;
        }

        public OffAxisCamerasData OffAxisData = new OffAxisCamerasData();

        [System.Serializable]
        public class OffAxisCamerasData
        {
            public int unscaledCanvasWidth;
            public int unscaledCanvasHeight;
            public List<OffAxisCameraData> cameras = new List<OffAxisCameraData>();

            public float qualityScale = 1.0f;
            public int CanvasWidth => Mathf.RoundToInt(unscaledCanvasWidth * qualityScale);
            public int CanvasHeight => Mathf.RoundToInt(unscaledCanvasHeight * qualityScale);
        }

        [System.Serializable]
        public class OffAxisCameraData
        {
            public string id;
            public Vector4 roi;
            public Vector3 lowerLeft;
            public Vector3 lowerRight;
            public Vector3 upperLeft;
        }
    }

    public class IglooCamera : MonoBehaviour
    {

        public RenderTexture finalTexture;
        protected EffectsChain effectsChain;

        public bool useWarpAndBlend = false;

        public bool debug = false;

        protected float ncp = 0.1f;
        protected float fcp = 100f;

        [Range(-0.1f, 0.1f)]
        public float ipd = 0.06f;

       
        public virtual void Setup(IglooCameraData data, GameObject templateCamera = null)
        {
            ncp = data.nearClipPlane; 
            fcp = data.farClipPlane;
            ipd = data.stereoEyeDistance;
            Application.targetFrameRate = data.FPSTarget;
        }

        public virtual void SetIPD(float val)
        {

        }

        private void Update()
        {

        }

        private void OnRenderImage(RenderTexture source, RenderTexture destination)
        {
            if (debug) Graphics.Blit(finalTexture, destination);
        }

    }
}



