using UnityEngine;
using UnityEngine.Rendering;
#if IGLOO_URP || IGLOO_URP_13
using UnityEngine.Rendering.Universal;
#endif

namespace Igloo.Common
{
#if IGLOO_URP || IGLOO_URP_13
    public class BlitMaterialFeature : ScriptableRendererFeature
    {
        class RenderPass : ScriptableRenderPass
        {
            private string profilingName;
            private Material material;
            private int materialPassIndex;
#if IGLOO_URP
            private RenderTargetIdentifier sourceID;
            private RenderTargetHandle renderTarget;
#endif
#if IGLOO_URP_13
            private RTHandle sourceID;
            private RTHandle renderTarget;
#endif

            public RenderPass(string profilingName, Material material, int passIndex) : base()
            {
                this.profilingName = profilingName;
                this.material = material;
                this.materialPassIndex = passIndex;

#if IGLOO_URP
                renderTarget.Init("_TempBlitMaterialTexture");
#endif
#if IGLOO_URP_13
                renderTarget = RTHandles.Alloc("_TempBlitMaterialTexture", name: "_TempBlitMaterialTexture");
#endif
            }
#if IGLOO_URP_13
            public void SetSource(RTHandle source)
#endif
#if IGLOO_URP
            public void SetSource(RenderTargetIdentifier source)
#endif
            {
                this.sourceID = source;
            }

            public override void Execute(ScriptableRenderContext context, ref RenderingData renderingData)
            {
                CommandBuffer cmd = CommandBufferPool.Get(profilingName);

                RenderTextureDescriptor cameraTextureDesc = renderingData.cameraData.cameraTargetDescriptor;
                cameraTextureDesc.depthBufferBits = 0;
#if IGLOO_URP
                cmd.GetTemporaryRT(renderTarget.id, cameraTextureDesc, FilterMode.Bilinear);
                Blit(cmd, sourceID, renderTarget.Identifier(), material, materialPassIndex);
                Blit(cmd, renderTarget.Identifier(), sourceID);

#endif
#if IGLOO_URP_13
                RenderingUtils.ReAllocateIfNeeded(ref renderTarget, cameraTextureDesc, FilterMode.Bilinear);
                Blit(cmd, sourceID, renderTarget, material, materialPassIndex);
                Blit(cmd, renderTarget, sourceID);
#endif



                context.ExecuteCommandBuffer(cmd);
                CommandBufferPool.Release(cmd);
            }

            public override void FrameCleanup(CommandBuffer cmd)
            {
#if IGLOO_URP
                cmd.ReleaseTemporaryRT(renderTarget.id);
#endif
#if IGLOO_URP_13
                cmd.ReleaseTemporaryRT(Shader.PropertyToID(renderTarget.name));
#endif
            }
        }

        [System.Serializable]
        public class Settings
        {
            public Material material;
            public int materialPassIndex = -1;
            public RenderPassEvent renderEvent = RenderPassEvent.AfterRenderingOpaques;
        }

        [SerializeField] private Settings settings = new Settings();

        private RenderPass renderPass;

        public Material Material
        {
            get { return settings.material; }
            set { settings.material = value;  }
        }

        public override void Create()
        {
            this.renderPass = new RenderPass(name, settings.material, settings.materialPassIndex);
            renderPass.renderPassEvent = settings.renderEvent;
        }

        public override void AddRenderPasses(ScriptableRenderer renderer, ref RenderingData renderingData)
        {
            // Only render on the camera tagged WarpBlendOutputCamera
            if (renderingData.cameraData.camera.CompareTag("WarpBlendOutputCamera"))
            {
#if IGLOO_URP
                renderPass.SetSource(renderer.cameraColorTarget);
#endif
#if IGLOO_URP_13
                renderPass.SetSource(renderer.cameraColorTargetHandle);

#endif
                renderer.EnqueuePass(renderPass);
            }
        }
    }
#else
            public class BlitMaterialFeature : MonoBehaviour {
    // This class is not used when not running URP
    }
#endif
}

